package de.usu.bs.examples.addressbook;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Properties;
 
//jms stuff
import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.io.IOException;
 
public class WildflyJMSQueueSender {
  public final static String JMS_CONNECTION_FACTORY_JNDI="jms/RemoteConnectionFactory";
  public final static String JMS_QUEUE_JNDI="jms/queue/addressbook";
  public final static String JMS_USERNAME="jmsuser";       //  The role for this user is "guest" in ApplicationRealm
  public final static String JMS_PASSWORD="secr3t";  
  public final static String WILDFLY_REMOTING_URL="http-remoting://localhost:8080";
 
  private QueueConnectionFactory qconFactory;
  private QueueConnection qcon;
  private QueueSession qsession;
  private QueueSender qsender;
  private Queue queue;
  private TextMessage msg;
  
	static String messageText = "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" "
			+ "xmlns:typ=\"http://bs.usu.de/services/examples/addressbook/types\" >"
			+ "<soapenv:Header/>" + "<soapenv:Body>" + "<typ:findAddress>" + "<typ:id>990</typ:id>"
			+ "<typ:fistname>John</typ:fistname>" + "<typ:lastname>Doe</typ:lastname>"
			+ "<!--Zero or more repetitions:-->" + "<typ:address>" + "<typ:id>999</typ:id>"
			+ "<typ:type>home</typ:type>" + "<typ:street>Shepards Lane 1</typ:street>"
			+ "<typ:postcode>12-mn-456</typ:postcode>" + "<typ:city>Brixton</typ:city>"
			+ "<typ:country>UK</typ:country>" + "</typ:address>" + "</typ:findAddress>" + "</soapenv:Body>"
			+ "</soapenv:Envelope>";

 
  public static void main(String[] args) throws Exception {
    InitialContext ic = getInitialContext();
    WildflyJMSQueueSender queueSender = new WildflyJMSQueueSender();
    queueSender.init(ic, JMS_QUEUE_JNDI);
    readAndSend(queueSender);
    queueSender.close();
  }
 
  public void init(Context ctx, String queueName) throws NamingException, JMSException {
    qconFactory = (QueueConnectionFactory) ctx.lookup(JMS_CONNECTION_FACTORY_JNDI);
     
    //  If you won't pass jms credential here then you will get 
    // [javax.jms.JMSSecurityException: HQ119031: Unable to validate user: null]    
    qcon = qconFactory.createQueueConnection(this.JMS_USERNAME, this.JMS_PASSWORD);   
     
    qsession = qcon.createQueueSession(false, Session.CLIENT_ACKNOWLEDGE);
    queue = (Queue) ctx.lookup(queueName);
    qsender = qsession.createSender(queue);
    msg = qsession.createTextMessage();
    qcon.start();
  }
 
  public void send(String message,int counter) throws JMSException {
    msg.setText(message);
    msg.setIntProperty("counter", counter);
    msg.setStringProperty("SOAPJMS_contentType", "text/xml");
    msg.setStringProperty("SOAPJMS_requestURI", "jms:jndi:addressbook.response");
    qsender.send(msg);
  }
 
  public void close() throws JMSException {
    qsender.close();
    qsession.close();
    qcon.close();
  }
 
  private static void readAndSend(WildflyJMSQueueSender wildFlyJmsQueueSender) throws IOException, JMSException {
    String line= messageText;
          wildFlyJmsQueueSender.send(line, 1);
          System.out.println("JMS Message Sent: \n"+line+ "\n");
  }
 
  private static InitialContext getInitialContext() throws NamingException {
     InitialContext context=null;
     try {
           Properties props = new Properties();
           props.put(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.naming.remote.client.InitialContextFactory");
           props.put(Context.PROVIDER_URL, WILDFLY_REMOTING_URL);   // NOTICE: "http-remoting" and port "8080"
           props.put(Context.SECURITY_PRINCIPAL, JMS_USERNAME);
           props.put(Context.SECURITY_CREDENTIALS, JMS_PASSWORD);
           //props.put("jboss.naming.client.ejb.context", true);
           context = new InitialContext(props); 
           System.out.println("\n\tGot initial Context: "+context);     
      } catch (Exception e) {
           e.printStackTrace();
      }
    return context;
  }
}